#!/bin/sh
####################################################################################
#  Title : SASDepotSubset.sh 
#  Version: 1.0
#  Release Date: FEB2010
#
#  Description: This script will copy an order from a SAS Software Depot and create
#               a smaller subset SAS Software Depot.
#
#  Output: This script will create one subset SAS Software Depot in the user specified directory.
#
#  Usage : ./SASDepotSubset.sh -i sasdirectory -o outputdirectory -v
#           -o     The order number to use in creating the subset SAS Software Depot
#           -i     The directory of the existing SAS Software Depot
#           -d     The directory where the subset SAS Software Depot will be written. 
#           -p     Input file containing the 12-Byte product names to be copied to the subset SAS Software Depot
#           -s     Operating systems to be included in the subset SAS Software Depot
#           -v     Verbose, echo log messages to the shell as well as logging them in $outputdirectory/SASDepotSubset.log
#
#  Note1: You should run this script with a userid that has read access to the existing SAS Software Depot
#         and write access to the output directory
#####################################################################################
set -f
umask 0022
###############################
# Describe our usage parameters
###############################
usage(){
  $ECHO "\nSASDepotSubset Version $myversion \n"
  $ECHO "USAGE:\t$1 -o OrderNumber -i InputSASDirectory -d outputDirectory -v <verbose> \n"  
  $ECHO "-o   The order number to extract from the existing SAS Software Depot (required)\n"        
  $ECHO "-i   The top level directory of the existing SAS Software Depot (required)\n"
  $ECHO "-d   The directory where this script will create a subset SAS Software Depot (required)"
  $ECHO "     If the output directory does not exist the script will attempt to create it.  The output directory "
  $ECHO "     must be an empty directory. \n"
  $ECHO "-p   This specifies an input file which contains the names of the products (12-Byte names) to be copied to the "
  $ECHO "     subset SAS Software Depot.  See example below for product file contents. \n"
  $ECHO "-s   This specifies operating systems to be included in the subset SAS Software Depot. (optional) "
  $ECHO "     If more than one operating system is specified, separate them with a space. Example: -s wx6 win s64 \n"  
  $ECHO "-v   Verbose, echo log messages to the shell as well as logging them in outputdirectory/SASDepotSubset.log. (optional) \n"
  $ECHO "OUTPUT: Along with the subset SAS Software Depot, this script also creates a log file in "
  $ECHO "        the same directory named SASDepotSubset.log. \n"
  $ECHO "        The script exits with 0 if no errors are encountered.  It exits with 1 if errors are encountered. \n"
  $ECHO "NOTE: This script should be executed by a userid that has read access to the existing SAS Software Depot"
  $ECHO "      and write access to the directory where the subset SAS Software Depot will be written.\n"
  $ECHO "WARNING: Do not use backslashes in any of the input parameters. Use forward slashes only. \n"
  $ECHO "Example contents of product file (SASDepotSubset.txt): "
  $ECHO "##################################################################### "
  $ECHO "# This file specifies the 12-Byte names of the SAS Subset Products "
  $ECHO "# that are eligible to be copied to a subset SAS Software Depot by the "
  $ECHO "# SASDepotSubset.sh tool. "  
  $ECHO "# "
  $ECHO "# You can comment out any products you do NOT want copied into "
  $ECHO "# the subset SAS Software Depot by putting # in the first column. "
  $ECHO "#################################################################### "
  $ECHO "clientconfig "
  $ECHO "dmine "
  $ECHO "dmineclient "
  $ECHO "dminethincfg "
  $ECHO "eguide "
  $ECHO "localesetmgr "
  $ECHO "msofficeint "
  $ECHO "tmclient "
  $ECHO "ttfontsvert \n"
  $ECHO "Example script usage: "
  $ECHO "SASDepotSubset.sh -o 012345 -i /existing/sas/depot/directory -d /new/subset/depot/directory -p SASDepotSubset.txt -s wx6 win s64 -v \n"
}

###################
# Log our actions
###################
log_write() {
  $ECHO "   $1" >> $OUTPUT_FILE
  if [ $verbose -eq 1 ]; then
    $ECHO "$1"
  fi  
}

#######################
# Check files on disk #
#######################
files_on_disk() {
  myfile="$1"
  exist=1
  empty=0
  directory=0
  if [ -d "$myfile" ]; then
     directory=1
  fi
    
  ## Checking that all files exist 
  if [ ! -r "$myfile" ];then
    log_write "ERROR: $myfile does NOT exist."
    exist=0
  else
    if [ $directory -eq 1 ]; then  
      ## Does the directory have any files? 
      length=`ls -1 | wc -l`
    
    else
      # $1=permissions
      # $3=owner
      # $4=group
      # $5=length
      filestuff=`ls -ld "$myfile" | $AWK '{print $1 " " $2 " " $3 " " $4 " " $5}'`
      length=`$ECHO "$filestuff" | $AWK '{print $5}'`
    fi
    
    if [ $length -eq 0 ]; then
      #log_write "NOTE: $myfile is empty."
      empty=1;
    fi
  fi 
}

###############################################
# Check the altsku to see if it is one of
# the mandatory 12bytes we want to process
# and that it is on the right operating system
###############################################
check_sku(){ 
  mysku="$1"
  goodsku=0
  goodsys=0
  foundproduct=""
  under="__"
  oneunder="_"
 
  if [ "$systems" != "ALL" ]; then     
    for s in $systems
    do
      if [ `echo $mysku | grep -ic $under$s$under` -gt 0 ]; then
	goodsys=1
	#####################################################################
	# If we found the operating system, remove it from the not found list
	##################################################################### 	
	notfoundsystems=`$ECHO $notfoundsystems | sed s/$s//g` 	
      fi	
    done 
  else
    goodsys=1  
  fi
  ############################################################################################
  # If this altsku has the right operating system, check to see if we have the right product
  ###########################################################################################
  if [ $goodsys -eq 1 ]; then
    ################################################################################
    # Check to see if this altsku is one from our required list.
    ################################################################################ 
    for product in $requiredproducts
    do   
      if [ `echo $mysku | grep -ci $product$under` -gt 0 ]; then
        ### We might have a match, see if it starts in the first column ###
	tempsku=`$ECHO $mysku | $AWK '$1 ~/^'$product$under'/'`
	  if [ "$tempsku" = "$mysku" ]; then
          goodsku=1
	  foundproduct="$product"
	fi
      fi	
    done

    ################################################################################
    # If we have not yet found a goodsku from the required list above,
    # check to see if the altsku is one of the optional 12-Bytes the user specified.
    ################################################################################ 
    if [ $goodsku -eq 0 ]; then
      for product in $productlist
      do
	if [ `echo $mysku | grep -ci $product$under` -gt 0 ]; then
          ### We might have a match, see if it starts in the first column ###
	  tempsku=`$ECHO $mysku | $AWK '$1 ~/^'$product$under'/'`	  
	  if [ "$tempsku" = "$mysku" ]; then
            goodsku=1
	    productfound=1
	    foundproduct="$product"
	  fi	
	fi	
      done
    fi  
  fi

  ########################################################################################
  # If a goodsku, remove it from the not found list 
  # If not a goodsku, add it to the excludedaltskus list
  ####################################################################################### 
  if [ $goodsku -eq 1 ]; then
    notfoundproductlist=`$ECHO $notfoundproductlist | sed s/" $foundproduct "/" "/g`       
  else
    excludedaltskus="$excludedaltskus $mysku"       
  fi 
}
############################################
# Create a directory
############################################
makedir(){ 
  `mkdir $1`  
  if [ ! -d "$1" ]; then
    log_write "ERROR: Not able to create directory $1."
    log_write "ERROR: Processing has stopped."
    cleanup
  #else
    #log_write "CREATED: $1"  
  fi  
  dircnt=`expr $dircnt + 1` 
}

############################################
# Copy a file
############################################
copyfile(){ 
  `cp $1 $2`  
  touch -r $1 $2
  filecnt=`expr $filecnt + 1`  
}

############################################
# Delete a file
############################################
deletefile(){ 
  `rm $1`  
  deletefilecnt=`expr $deletefilecnt + 1` 
}

############################################
# Delete a directory recursively
############################################
deletedir(){ 
  `rm -R $1`  
  deletedircnt=`expr $deletedircnt + 1` 
}

############################################
# Make a file
############################################
makefile() {
  `touch $1`
  if [ ! -f "$1" ]; then
    log_write "ERROR: Not able to create file $1."
    log_write "ERROR: Processing has stopped."
    cleanup
  #else
    #log_write "CREATED: $1"  
  fi
}
############################################
# Update the output files to log our results
# and clean up stuff
############################################
cleanup(){
  ### Log the messages ###   
  log_write "$dircnt directories were created"
  log_write "$filecnt files were copied"  
  log_write "$deletefilecnt files were deleted"
  log_write "$deletedircnt directories were deleted"  
  log_write "Log has been recorded in: $OUTPUT_FILE"
  exit 1
}

################
# Main Program #
################
osname=`uname -s`
if [ $osname = "SunOS" ]; then
  AWK=nawk
else
  AWK=awk
fi

if [  $osname = "Linux" ]; then
  ECHO='echo -e'
else
  ECHO=echo
fi

###################################
# Initialize our counters and text
###################################
filecnt=0
deletefilecnt=0
deletedircnt=0
dircnt=0
under="__" 
oneunder="_"
myversion="2.3"
  
############################
# Parse the input parameters
############################
set -- `getopt i:o:d:p:s:v: "$*"`
if [ $? -ne 0 ];then
  usage `basename $0`
  exit 1
fi
  
if [ $1 != "-o" ] && [ $1 != "-i" ] && [ $1 != "-d" ] && [ $1 != "-p" ] && [ $1 != "-v" ]; then
  usage `basename $0`
  exit 1
fi

myparm=0
indir=""
outdir=""
order=""
productfile=""
systems=" "
verbose=0
suffix="_subset"

for i in $*
do
  if [ "$i" = "-o" ]; then
    myparm=1
  elif [ "$i" = "-i" ]; then
    myparm=2
  elif [ "$i" = "-d" ]; then
    myparm=3
  elif [ "$i" = "-p" ]; then
    myparm=4
  elif [ "$i" = "-s" ]; then
    myparm=5   
  elif [ "$i" = "-v" ]; then
    verbose=1
    myparm=6    
  fi  
  if [ $myparm -eq 1 -a "$i" != "-o" -a "$i" != "--" ]; then
    order=`$ECHO $i`
  elif [ $myparm -eq 2 -a "$i" != "-i" -a "$i" != "--" ]; then
    indir=`$ECHO $i` 
  elif [ $myparm -eq 3 -a "$i" != "-d" -a "$i" != "--" ]; then
    outdir=`$ECHO $i`     
  elif [ $myparm -eq 4 -a "$i" != "-p" -a "$i" != "--" ]; then
    productfile=`$ECHO $i` 
  elif [ $myparm -eq 5 -a "$i" != "-s" -a "$i" != "--" ]; then
    systems=`$ECHO $systems $i`     
  fi    
done

##########################################
# Do some checking on the parameter values
##########################################
if [ -z "$indir" ]; then
  $ECHO "ERROR: No Input SAS Software Depot Directory was specified."
  $ECHO "ERROR: Processing has stopped"
  usage
  exit 1
fi
if [ ! -d "$indir" ]; then
  $ECHO "ERROR: Invalid Input SAS Software Depot Directory: $indir"
  $ECHO "ERROR: Processing has stopped"
  usage
  exit 1
fi 

if [ -z "$outdir" ]; then
    $ECHO "ERROR: No Output Directory was specified."
    $ECHO "ERROR: Processing has stopped"
    usage
    exit 1
fi
if [ ! -d "$outdir" ]; then
  # Try to create it
  makedir "$outdir"
  if [ ! -d "$outdir" ]; then
    $ECHO "ERROR: Invalid Output Directory: $outdir"
    $ECHO "ERROR: Could not create $outdir"
    $ECHO "ERROR: Processing has stopped"
    usage
    exit 1
  fi  
fi
if [ -z "$productfile" ]; then
  $ECHO "ERROR: No product file was specified."
  $ECHO "ERROR: Processing has stopped"
  usage
  exit 1
fi
if [ ! -f "$productfile" ]; then
  $ECHO "ERROR: The specified product file does not exist: $productfile."
  $ECHO "ERROR: Processing has stopped"
  usage
  exit 1
fi
if [ ! -s "$productfile" ]; then
  $ECHO "ERROR: An empty product file was specified: $productfile"
  $ECHO "ERROR: Processing has stopped"
  usage
  exit 1
fi

#########################################
# Make sure the output directory is empty
#########################################
rc=`ls -lA $outdir | grep -ic :`
if [ $rc -gt 0 ]; then
  $ECHO "ERROR: Output directory is not empty: $outdir "
  $ECHO "ERROR: Processing has stopped."
  usage
  exit 1
fi 

#############################
# Initialize our output log
############################
OUTPUT_FILE="$outdir/SASDepotSubset.log" 
$ECHO `date` > $OUTPUT_FILE
log_write "SASDepotSubset Version $myversion"
log_write "Order number = $order" 
log_write "Input SAS Software Depot = $indir" 
log_write "The Subset SAS Software Depot will be created in $outdir"
log_write "List of products will be read from $productfile"
log_write "Log file has been created in $OUTPUT_FILE"
log_write "   "

if [ "$systems" != " " ]; then
  log_write "Only the following operating systems will be included in the subset SAS Software Depot, if they are found in the master SAS Software Depot: "
  systems=`$ECHO $systems | tr '[:upper:]' '[:lower:]'`
  for sys in "$systems"
  do
    log_write "$sys"
  done 
  ###########################################################################################
  # Prime the list of not found systems, we remove them from this list if we copy them later
  ###########################################################################################
  notfoundsystems="$systems "   
  ###############################################################################################
  # Add prt and sps to the list of system delimeters that we will copy. 
  # __prt__ and __sps__ files are required regardless of operating systems specified by the user.
  ##############################################################################################
  systems="prt sps $systems "
else
  log_write "All operating systems found in the master SAS Software Depot will be included in the subset SAS Software Depot." 
  systems="ALL" 
fi

########################################
# Create the list of required products
########################################
requiredproducts="cfgwizard deploywiz javaruntime mit ordersummary quickstart sampleplans sdwstartup sid_files soi "

##################################################
# Verify the indir is really a sas software depot
##################################################
files_on_disk "$indir/order_data"
if [ $exist -eq 0 ]; then
  log_write "ERROR: Input SAS Software Depot does not appear to be a 9.2 SAS Software Depot or may be corrupt."
  log_write "ERROR: $indir/order_data directory does not exist."
  log_write "ERROR: Processing has stopped."
  cleanup
fi 

###################################
# Begin processing the order number
###################################
if [ ! -d "$indir/order_data/$order" ]; then
  log_write "ERROR: Order $order does not exist in specified SAS Software Depot $indir." 
  log_write "ERROR: No subset SAS Software Depot will be created."
  log_write "ERROR: Processing has stopped."
  cleanup
fi 

if [ -d "$indir/order_data/$order$suffix" ]; then
  log_write "ERROR: The specified input SAS Software Depot appears to already be a subset SAS Software Depot." 
  log_write "ERROR: No subset SAS Software Depot will be created."
  log_write "ERROR: Processing has stopped."
  cleanup
fi  

##################################################################################################
# We have a valid order number so proceed.
# Check to see if the master SAS Software Depot is a 9.2 SAS Software Depot by checking order.xml
#################################################################################################
files_on_disk "$indir/order_data/$order/order.xml"
if [ $exist -eq 0 -o $empty -eq 1 ]; then
  log_write "ERROR: Input SAS Software Depot does not appear to be a 9.2 SAS Software Depot or may be corrupt."
  log_write "ERROR: $indir/order_data/$order/order.xml does not exist or is empty."
  log_write "ERROR: Processing has stopped."
  cleanup
fi 

##################################################################################################
# Find the version of this depot from order.xml
#################################################################################################
depotversion=0
depotversion=`grep -i "depot_version" "$indir/order_data/$order/order.xml" | $AWK -F\" '{print $2}'`
if [ -z $depotversion ]; then
  depotversion=0
fi  

#######################################################
# Read the list of optional products we will be copying
#######################################################
productfound=0
grep -v '#' "$productfile" > "$outdir/myproducts.$$"
productlist=""
notfoundproductlist=" "
excludedaltskus=" "
haveproducts=0
amoselect=0
mapcenterselect=0
biwebptsmsspselect=0
log_write "   "
log_write "$productfile contains the following products: "
for product in `cat "$outdir/myproducts.$$"`
do
  gotit=`$ECHO $productlist | grep -icw $product`
  if [ $gotit -eq 0 ]; then
    product=`$ECHO $product | tr '[A-Z]' '[a-z]'`  
    productlist="$productlist $product"
    haveproducts=1
    if [ `echo $product | grep -icw msofficeint` -gt 0 ]; then
      amoselect=1
    fi
    if [ `echo $product | grep -icw mapcenter` -gt 0 ];  then
      mapcenterselect=1
    fi  
    if [ `echo $product | grep -icw biwebptsmssp` -gt 0 ];  then
      biwebptsmsspselect=1
    fi      
  fi
done    

### Change to all lower case ###
productlist=`$ECHO $productlist | tr '[A-Z]' '[a-z]'`
### Remove Carriage Returns ###
productlist=`$ECHO $productlist | tr -d '\015'`
### Remove null characters ###
productlist=`$ECHO $productlist | tr -d '\0'`
log_write "$productlist"
log_write "   "

#############################################################################
### If they chose msofficeint but not mapcenter, force the mapcenter install
### We will modify the install later.
#############################################################################
if [ $depotversion -lt 2 ]; then
  if [ $amoselect -eq 1 -a $mapcenterselect -eq 0 ]; then
    productlist="$productlist mapcenter"
  fi  
fi
if [ $depotversion -ge 2 ]; then
  if [ $amoselect -eq 1 -a $biwebptsmsspselect -eq 0 ]; then
    productlist="$productlist biwebptsmssp"
  fi  
fi

# Prime the list of not found products, we remove them as we find them later
notfoundproductlist="_beginlist_ $productlist cfgwizard deploywiz javaruntime mit ordersummary quickstart sampleplans sdwstartup sid_files soi _endlist_"
if [ $haveproducts -eq 0 ]; then
  log_write "ERROR: No products were specified in $productfile."
  log_write "ERROR: Processing has stopped."
  cleanup
fi
rm $outdir/myproducts.$$

######################################################################
# Make the order_data/$order_subset directory then recursively copy it
#####################################################################
makedir "$outdir/order_data"
makedir "$outdir/order_data/$order$suffix"
ls -A $indir/order_data/$order > "$outdir/orderfiles.$$"
for myfile in `cat "$outdir/orderfiles.$$"`
do
  ( cd $indir/order_data/$order ; tar cf - . ) | ( cd $outdir/order_data/$order$suffix ; tar xf - )
  filecnt=`expr $filecnt + 1` 
done
dircnt=`expr $dircnt + 1`
if [ ! -d "$outdir/order_data/$order$suffix" ]; then
  log_write "ERROR: Not able to create $outdir/order_data/$order$suffix directory."
  log_write "ERROR: Processing has stopped."
  cleanup
#else
  #log_write "COPIED: $outdir/order_data/$order$suffix."  
fi

################################################
# Massage a few things in the order.xml file
# Set number="$order_subset" and sa_depot="true"
################################################
sed "s/number=\"$order\"/number=\"$order$suffix\"/g" $outdir/order_data/$order$suffix/order.xml > $outdir/order_data/$order$suffix/order2.xml
sed "s/sa_depot=\"false\"/sa_depot=\"true\"/g" $outdir/order_data/$order$suffix/order2.xml > $outdir/order_data/$order$suffix/order.xml
rm $outdir/order_data/$order$suffix/order2.xml

####################################################
# Create the setup.dat and sassd.txt and cd.id files
####################################################  
makefile "$outdir/sassd.txt"
setup="$outdir/setup.dat"
makefile "$setup"
cdid="$outdir/cd.id"
makefile "$cdid"

#################################
# Create the directories we need 
#################################
makedir "$outdir/media_data"
makedir "$outdir/product_data"
makedir "$outdir/products"
makedir "$outdir/install_doc"
    
##################################
# Copy the depotsummary.html file 
##################################
copyfile "$indir/depotsummary.html $outdir"
    
########################################
# Copy the media_data/order_*.xml files 
########################################
xmldir=$indir/media_data
xmlfiles=`ls -A $xmldir | grep -i .xml | grep -i $order`

# copy all the xml files
for xmlfile in $xmlfiles
do
  copyfile "$indir/media_data/$xmlfile $outdir/media_data"  
  if [ ! -f "$outdir/media_data/$xmlfile" ]; then
    log_write "ERROR: Not able to copy $xmlfile to $outdir/media_data."
    log_write "ERROR: No subset SAS Software Depot has been created."
    cleanup
  #else
    #log_write "COPIED: $outdir/media_data/$xmlfile."  
  fi
done

############################################################################	
# Loop through the volsers from order.xml for our order number
# Start by getting the mediatype
############################################################################
orderxml="$indir/order_data/$order/order.xml"
esdm=`grep -ic 'mediatype="ESDM"' "$orderxml"`
dvdm=`grep -ic 'mediatype="DVDM"' "$orderxml"`
cdrm=`grep -ic 'mediatype="CDRM"' "$orderxml"`
esdm=`grep -ic 'mediatype="ESDM"' "$orderxml"`	
# Get the filenames for the appropriate mediatype;
if [ $esdm -gt 0 ]; then
  grep -i _es "$orderxml" | grep -in volser | $AWK -F\" '{print $2}' > "$outdir/volsers.$$"
elif [ $dvdm -gt 0 ]; then
  grep -i _ds "$orderxml" | grep -in volser | $AWK -F\" '{print $2}' > "$outdir/volsers.$$"
elif [ $cdrm -gt 0 ]; then
  grep -i _cd "$orderxml" | grep -in volser | $AWK -F\" '{print $2}' > "$outdir/volsers.$$"
else
  log_write "ERROR: $orderxml file is corrupt. Input SAS Software Depot may be corrupt. Recopy or redownload the SAS Software Depot."
  log_write "ERROR: Processing has stopped"
  cleanup
fi 

#############################################################
# Read the $indir/media_data/$volser.xml file for each volser
#############################################################
for volser in `cat "$outdir/volsers.$$"`
do
  #######################################################
  # Read the altsku and pkgpath for each volser.
  # We also add a line for each volser to the cd.id file.
  ####################################################### 
  foundit=`grep -c $volser "$cdid" | $AWK -F\t '{print $1}'`
    if [ $foundit -eq 0 ]; then
      $ECHO "$volser" >> "$cdid"
    fi
  ############################################
  # Check out the volserxml, if good then copy
  ############################################
  volserxml="$indir/media_data/$volser.xml"	
  files_on_disk "$volserxml"
  if [ $exist -eq 0 -o $empty -eq 1 ];then
    log_write "ERROR: $volserxml is empty or does not exist. Input SAS Software Depot may be corrupt. Recopy or redownload the SAS Software Depot."
    log_write "ERROR: No subset SAS Software Depot has been created."
    cleanup 
  else
    copyfile "$volserxml $outdir/media_data/$volser.xml"
    if [ ! -f "$outdir/media_data/$volser.xml" ]; then
      log_write "ERROR: Not able to copy $volserxml to $outdir/media_data."
      log_write "ERROR: No subset SAS Software Depot has been created."
      cleanup
    #else
      #log_write "COPIED: $outdir/media_data/$volser.xml."  
    fi   
    
    ######################################################
    # Generate a listing of altskus then loop through them
    ######################################################
    grep -i 'product alt_sku=' "$volserxml" | sed 's/format="standard"//g' | $AWK -F\" '{print $2 "---" $4}' > "$outdir/altskus.$$"   
    for myline in `cat "$outdir/altskus.$$"`
    do
      ###################################################################
      # Read all the filelist.xml files to get the files we need to copy.
      ###################################################################
      # Assign both $altsku and $pkgpath for the current line
      altsku=`$ECHO $myline | $AWK -F\--- '{print $1}'`
      pkgpath=`$ECHO $myline | $AWK -F\--- '{print $2}'`	

      #############################################################################
      # Check to see if this altsku is one of the subset 12bytes that we will copy.
      # If good altsku, check the filelist.xml file before continuing.
      #############################################################################
      check_sku "$altsku"	  
      if [ $goodsku -eq 1 ]; then    	
        files_on_disk "$indir/product_data/$altsku/filelist.xml"
        if [ $exist -eq 0 -o $empty -eq 1 ]; then
	  log_write "ERROR: $indir/product_data/$altsku/filelist.xml is empty or does not exist. Input SAS Software Depot may be corrupt. Recopy or redownload the SAS Software Depot."
	  log_write "ERROR: No subset SAS Software Depot has been created."
	  cleanup 
        fi  
        skupkg=0
        skudat=0
 
        #####################################################################################
        # Set the filelist.xml to stdin so we can read the contents of filelist.xml properly
	####################################################################################
        exec 9<&0 < "$indir/product_data/$altsku/filelist.xml"
        while read line
        do
          myvar1=`$ECHO $line | $AWK '{print $1}'`
          myvar2=`$ECHO $line | $AWK '{print $2}'`

          if [ $myvar1 = "<skupkg" ]; then
            skupkg=1
          elif [ $myvar1 = "</skupkg>" ]; then
            skupkg=0
          elif [ $myvar1 = "<skudat" ]; then
            skudat=1
          elif [ $myvar1 = "</skudat>" ]; then
            skudat=0   
          fi 
          #########################################################
          # Now we know if we are dealing with a skupkg or skudat.
	  # Copy the file to the appropriate directory.
	  ########################################################
          if [ $myvar1 = "<file" ]; then    
            file=`$ECHO "$myvar2" | $AWK -F\" '{print $2}'`
            if [ $skupkg -eq 1 ]; then
              files_on_disk "$indir/$pkgpath/$file"	  
	      if [ $exist -eq 1 ]; then
		# Make the directory if we haven't already
		if [ ! -d "$outdir/$pkgpath" ]; then
		  makedir "$outdir/$pkgpath"
		fi  
		if [ $directory -eq 1 ]; then
		  makedir "$outdir/$pkgpath/$file"
		   #log_write "CREATED: directory $outdir/$pkgpath/$file"
		else   
		  copyfile "$indir/$pkgpath/$file $outdir/$pkgpath/$file"
		  #log_write "COPIED: $indir/$pkgpath/$file    TO    $outdir/$pkgpath/$file"		  
		fi  		
	      fi  
            elif [ $skudat -eq 1 ]; then
              files_on_disk "$indir/product_data/$altsku/$file"	      
	      if [ $exist -eq 1 ]; then		
		if [ ! -d "$outdir/product_data/$altsku" ]; then
		  makedir "$outdir/product_data/$altsku"
		fi  
		if [ $directory -eq 1 ]; then
		  makedir "$outdir/product_data/$altsku/$file"
		  #log_write "CREATED: directory $outdir/product_data/$altsku/$file"
		else   
		  copyfile "$indir/product_data/$altsku/$file $outdir/product_data/$altsku/$file"
		  #log_write "COPIED: $indir/product_data/$altsku/$file    TO    $outdir/product_data/$altsku/$file"		  
		fi  
	      fi  	      
            fi #skupkg eq 1 elif skudat eq 1
          fi #myvar1 = <file
        done #while read line
        exec 0<&9 9<&- 
      fi  #goodsku=1
    done #myline in outdir/altskus
  fi #volserxml not empty and exists
done  #for volser in volsers    
##########################################
# Initialize then load the setup.dat file
##########################################
$ECHO "[default]" > $setup
####################################
# Fill in the default deploywiz info
####################################
deploywiz=`ls $outdir/products | grep -i deploywiz__`
$ECHO "inifile=products/$deploywiz/deploywiz.ini" >> $setup
$ECHO "  " >> $setup
$ECHO "[properties]" >> $setup
twelvebyte=`$ECHO $deploywiz | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/' | $AWK -F\__ '{print $1}'`
host=`$ECHO $deploywiz | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/' | $AWK -F\__ '{print $3}'`
myunderscore='_'
pkg="PKG"
$ECHO "$twelvebyte$myunderscore$host$myunderscore$pkg=products/$deploywiz" >> $setup
########################################################
# Gather the javaruntime altskus then loop through them
#######################################################
ls $outdir/products | grep -i javaruntime__ > "$outdir/setuptemp.$$"
# Loop through the javaruntime altskus and fill in the properties
for altsku in `cat "$outdir/setuptemp.$$"`
do
  twelvebyte=`$ECHO $altsku | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/' | $AWK -F\__ '{print $1}'`
  host=`$ECHO $altsku | sed 'y/abcdefghijklmnopqrstuvwxyz/ABCDEFGHIJKLMNOPQRSTUVWXYZ/' | $AWK -F\__ '{print $3}'`
  $ECHO "$twelvebyte$myunderscore$host$myunderscore$pkg=products/$altsku" >> $setup
done #for altsku in setuptemp

############################################################
# Set the permissions to 0755 for all files and directories
###########################################################
chmod -R 0755 $outdir

###########################################################################################
# The depot is created so we now process any foundation products which were excluded.
# First, check to see if they excluded the 'sasinstall' altsku, if they did then
# just delete the order_data/order_subset/sasinstall__* altsku directory and we are done.
# If sasinstall was included then we have to edit toc and/or ini files.
#
# Keep only sasinstall__*__os__* directories for the os we installed.
# If we excluded sasinstall all together, delete all the sasinstall__* directories.
# No need to know if we are working with a unix or windows altsku.  Just test the existence 
# of CMTC00EN.TXT  and/or .ini files to know how to deal with them.
##########################################################################################
sasinstalldir=$outdir/order_data/$order$suffix
sasinstallskus=`ls -A $sasinstalldir | grep -i sasinstall__`

# delete all the sasinstall__* directories from $sasinstalldir that are excluded altskus
  for installsku in $sasinstallskus
  do
    if [ `echo $excludedaltskus | grep -icw $installsku` -gt 0 ]; then
      #log_write "EXCLUDED: $installsku"
      deletedir "$sasinstalldir/$installsku"
      #log_write "DELETED: $sasinstalldir/$installsku"
    fi  
  done  

# refresh the list of sasinstall altskus
sasinstallskus=`ls -A $sasinstalldir | grep -i sasinstall__`
colon=":"

# If there are any remaining sasinstall skus, process the toc or ini files
for installsku in $sasinstallskus
do
  # Windows?
  if [ -f $sasinstalldir/$installsku/sas/core/sasinst/data/base.ini ]; then
    inifiles=`ls -A $sasinstalldir/$installsku/sas/core/sasinst/data | grep .ini`
    inifiles=" $inifiles "
    for excludedsku in $excludedaltskus
    do
      #log_write "EXCLUDED: $excludedsku"
      for inifile in $inifiles
      do
        if [ -r $sasinstalldir/$installsku/sas/core/sasinst/data/$inifile ]; then
          if [ `grep -ic Altsku=$excludedsku $sasinstalldir/$installsku/sas/core/sasinst/data/$inifile` -gt 0 ]; then
            #log_write "DELETED: $sasinstalldir/$installsku/sas/core/sasinst/data/$inifile"
            deletefile "$sasinstalldir/$installsku/sas/core/sasinst/data/$inifile"
	    inifiles=`$ECHO $inifiles | sed s/" $inifile"/" "/` 
	  fi
	fi  
      done	 
    done
  fi
    
  # Unix?
  if [ -f $sasinstalldir/$installsku/CMTC00EN.TXT ]; then
    touch "$sasinstalldir/$installsku/CMTC00EN.$$.TXT"
    for excludedsku in $excludedaltskus
    do
        #log_write "EXCLUDED: $excludedsku"
	grep -iv $excludedsku$colon "$sasinstalldir/$installsku/CMTC00EN.TXT" > "$sasinstalldir/$installsku/CMTC00EN.$$.TXT"	
	cp "$sasinstalldir/$installsku/CMTC00EN.$$.TXT" "$sasinstalldir/$installsku/CMTC00EN.TXT" 
    done  
    rm "$sasinstalldir/$installsku/CMTC00EN.$$.TXT"   
  fi
done

################################################################################################################
# 9.2: AMO and MAPCENTER processing
# Process our special case where msofficeint (amo) is selected for copy but mapcenter is not.
# In this case we only want the product_data files for mapcenter so delete the product/mapcenter* directories.
# Modify product.xml and filelist.xml accordingly.  
# 9.3: AMO and BIWEBPTSMSSP processing
# Process our special case where msofficeint (amo) is selected for copy but biwebptsmssp is not.
###############################################################################################################
if [ $amoselect -eq 1 -a $mapcenterselect -eq 0  -a $depotversion -lt 2 ]; then
  ######################################################
  # Delete the products/mapcenter* directories and files
  ######################################################
  highmapdir=$outdir/products
  mapdirs=`ls -A $highmapdir | grep -i mapcenter`
  for mapdir in $mapdirs
  do
    deletedir "$outdir/products/$mapdir"
  done  

  # Find the product_data/mapcenter* directories we need to process
  highmapdir=$outdir/product_data
  mapdirs=`ls -A $highmapdir | grep -i mapcenter`

  # process the filelist.xml and product.xml files in the mapcenter directories
  for mapdir in $mapdirs
  do
    #########################
    # Process the product.xml
    #########################
    if [ -f "$highmapdir/$mapdir/product.xml" ]; then
      ##########################################################################
      # Change all selectabe v="true" to selectable v="false"
      # Read product.xml line by line and write only the lines we wish to keep
      # We are getting rid of the <install>...</install> blocks 
      ##########################################################################     
      productxml="$highmapdir/$mapdir/product.xml"
      newproductxml="$highmapdir/$mapdir/productnew.xml"	
      files_on_disk "$productxml"
      if [ $exist -eq 0 -o $empty -eq 1 ];then
        log_write "ERROR: $productxml is empty or does not exist. Input SAS Software Depot may be corrupt. Recopy or redownload the SAS Software Depot."
      else
 
        #####################################################################################
        # Set the product.xml to stdin so we can read the contents of product.xml properly
	####################################################################################
        firstline=1
	install=0
        exec 9<&0 < "$productxml"
        while read line
        do
          myvar1=`$ECHO $line | $AWK '{print $1}'`
          myvar2=`$ECHO $line | $AWK '{print $2}'`

          if [ "$myvar1" = "<install" ]; then
            install=1
          elif [ "$myvar1" = "</install>" ]; then
            install=0   
	  else  
            if [ "$line" = '<selectable v="true"/>' ]; then    
              line='<selectable v="false"/>'
	    fi
	    if [ $install -eq 0 ]; then
	      if [ $firstline -eq 1 ]; then
		$ECHO $line > $newproductxml
		firstline=0
	      else
		$ECHO $line >> $newproductxml
	      fi  
	    fi  	     
	  fi
        done #while read line
        exec 0<&9 9<&- 
        files_on_disk "$newproductxml"	  
	if [ $exist -eq 1 ]; then
	  cp -f $newproductxml $productxml
	  rm $newproductxml
	fi
      fi
    fi

    ########################## 
    # Process the filelist.xml
    ##########################
    if [ -f "$highmapdir/$mapdir/filelist.xml" ]; then
      # Read the filelist.xml line by line and write only the lines we wish to keep
      # We are getting rid of the <file>...<.file> blocks between the <skupkg>...</skupkg> tags
      # Remove all deployment data from the product.xml for mapcenter     
      filelistxml="$highmapdir/$mapdir/filelist.xml"
      newfilelistxml="$highmapdir/$mapdir/filelistnew.xml"	
      files_on_disk "$filelistxml"
      if [ $exist -eq 0 -o $empty -eq 1 ];then
        log_write "ERROR: $filelistxml is empty or does not exist. Input SAS Software Depot may be corrupt. Recopy or redownload the SAS Software Depot."
      else
 
        #####################################################################################
        # Set the filelist.xml to stdin so we can read the contents of filelist.xml properly
	####################################################################################
        firstline=1
	skupkg=0
        exec 9<&0 < "$filelistxml"
        while read line
        do
          myvar1=`$ECHO $line | $AWK '{print $1}'`
          myvar2=`$ECHO $line | $AWK '{print $2}'`

          if [ "$myvar1" = "<skupkg" ]; then
            skupkg=1
	    $ECHO $line >> $newfilelistxml
          elif [ "$myvar1" = "</skupkg>" ]; then
            skupkg=0 
          fi 
          #################################################################
          # Now we know if we are dealing with a skupkg block
	  ################################################################
	  if [ $skupkg -eq 0 ]; then
	    if [ $firstline -eq 1 ]; then
	      $ECHO $line > $newfilelistxml
	      firstline=0
	    else
	      $ECHO $line >> $newfilelistxml
	    fi    
	  fi  
        done #while read line
        exec 0<&9 9<&- 
        files_on_disk "$newfilelistxml"	  
	if [ $exist -eq 1 ]; then
	  cp -f $newfilelistxml $filelistxml
	  rm $newfilelistxml
	fi
      fi
    fi    
  done
  log_write "NOTE: Special processing has been performed on mapcenter since msofficeint was selected but mapcenter was not."
fi  

if [ $amoselect -eq 1 -a $biwebptsmsspselect -eq 0 -a $depotversion -ge 2 ]; then
  log_write "NOTE: The biwebptsmsspselect product has been added to the list of products to copy to this subset depot because it is required with msofficeint."
fi

###################################
# Log some more info then clean up
###################################
if [ $productfound -eq 0 ]; then
  log_write "WARNING: None of the selected products specified in $productfile were found in the source SAS Software Depot." 
  log_write "WARNING: The SAS Software Depot was created but contains no deployable products."
fi
if [ "$notfoundproductlist" != "_beginlist_ _endlist_" -a "$systems" != "ALL" ]; then
  notfoundproductlist=`$ECHO $notfoundproductlist | sed s/"_beginlist_"/" "/g` 
  notfoundproductlist=`$ECHO $notfoundproductlist | sed s/"_endlist_"/" "/g` 
  log_write "WARNING: The following products were NOT found in the source SAS Software Depot for this order and the requested operating systems:" 
  log_write "$notfoundproductlist"
elif [ "$notfoundproductlist" != "_beginlist_ _endlist_" ]; then
  log_write "WARNING: The following products were NOT found in the source SAS Software Depot:" 
  log_write "$notfoundproductlist"  
fi

if [ "$notfoundsystems" != "" -a "$systems" != "ALL" ]; then
  log_write "WARNING: The following operating systems were NOT found in the source SAS Software Depot for this order: $notfoundsystems" 
  log_write "WARNING: This depot may be unusable."
fi

log_write "$dircnt directories were created"
log_write "$filecnt files were copied"
log_write "$deletefilecnt files were deleted"
log_write "$deletedircnt directories were deleted"
errcnt=`grep -ic ERROR: $OUTPUT_FILE` 
log_write "$errcnt errors were encountered"
rm $outdir/altskus.$$
rm $outdir/volsers.$$	
rm $outdir/setuptemp.$$
rm $outdir/orderfiles.$$

################################
# If there are no errors, exit 0
################################
if [ $errcnt -gt 0 ]; then
  log_write "exit 1"
  exit 1
else
  log_write "exit 0"
  exit 0
fi    






